@@ -4,13 +4,16 @@
 #   Copyright (C) 2001-2004 Brian Elliott Finley
 #   Copyright (C) 2002 Dann Frazier <dannf@dannf.org>
 #
-#       $Id: Common.pm,v 1.40.2.1 2005/02/07 17:57:59 brianfinley Exp $
+#   $Id: Common.pm 3296 2005-12-15 13:19:42Z arighi $
 #
 #   2004.08.10  Brian Elliott Finley
 #   - where_is_my_efi_dir: remove /boot/efi/*/ from find path, but still 
 #     include /boot/efi.  find would fail if no dir existed below /boot/efi/.
 #   - where_is_my_efi_dir: redirect stderr to /dev/null so users don't see 
 #     confusing "no such file or directory" messages.
+#   2004.10.07  Brian Elliott Finley
+#   - add support for "mac" partition types in save_partition_information()
+#
 
 package SystemImager::Common;
 
@@ -35,6 +38,7 @@ $VERSION = $version_number;
 #   is_devfs_client
 #   numerically
 #   save_filesystem_information
+#   save_lvm_information -AR-
 #   save_partition_information
 #   where_is_my_efi_dir
 #   which
@@ -343,7 +347,7 @@ sub save_partition_information {
 
             chomp;
 
-            if (($label_type eq "gpt") || ($label_type eq "bsd")) {
+            if (($label_type eq "gpt") || ($label_type eq "bsd") || ($label_type eq "mac")) {
 
               # Unfortunately, parted doesen't produce it's output in a comma delimited
               # format, or even produce a n/a value such as "-" for fields that don't
@@ -355,22 +359,41 @@ sub save_partition_information {
               #
               # Sean Dague may have some better ideas on how to parse this output without
               # modifying parted. -BEF-
-              #
+
+              ### "gpt" sample ###
               # Disk geometry for /dev/sdb: 0.000-17366.445 megabytes
               # Disk label type: GPT
               #          10        20        30        40        50        60        70        80
               # 12345678901234567890123456789012345678901234567890123456789012345678901234567890
-              #     5           12          12          12                    22
+              #     v5          v12         v12         v12                   v22
               # 1234512345678901212345678901212345678901212345678901234567890121 -> to the end
               # |    |           |           |           |                     |
               # Minor    Start       End     Filesystem  Name                  Flags
               # 1          0.017     20.000                                    boot, lba
               # 2         21.000     40.000  FAT                               lba
               # 3         41.000  17366.429                                    lba
+              #
+              #
+              ### "mac" sample ###
+              # finley@imageserver:~/si.v3_4_x.ppc% sudo parted -s -- /dev/sda print
+              # Disk geometry for /dev/sda: 0.000-152627.835 megabytes         
+              # Disk label type: mac
+              #          10        20        30        40        50        60        70        80
+              # 12345678901234567890123456789012345678901234567890123456789012345678901234567890
+              #     v5          v12         v12         v12                   v22
+              # 1234512345678901212345678901212345678901212345678901234567890121 -> to the end
+              # |    |           |           |           |                     |
+              # Minor    Start       End     Filesystem  Name                  Flags
+              # 1          0.000      0.031              Apple                 
+              # 2          0.031      1.031  hfs         untitled              boot
+              # 3          1.031  50001.031  ext3        untitled              
+              # 4      50001.031  51001.031  ext3        untitled              
+              # 5      51001.031  52993.467  linux-swap  swap                  swap
+              # 6      52993.468 152627.835  ext3        untitled              
               ($minor, $startMB, $endMB, $fstype, $name, $flags) = unpack("A5 A12 A12 A12 A22 A*", $_);
 
-              # In the case of a gpt partition, they're all primary, and parted doesn't
-              # produce output to indicate this.
+              # In the case of a gpt, or mac partition, they're all primary, 
+              # and parted doesn't produce output to indicate this. -BEF-
               $partition_type = "primary";
 
             } elsif ($label_type eq "msdos") {
@@ -381,18 +404,20 @@ sub save_partition_information {
               # Disk label type: msdos
               #          10        20        30        40        50        60        70        80
               # 12345678901234567890123456789012345678901234567890123456789012345678901234567890
-              #     5           12          12        10          10
-              # 1234512345678901212345678901212345678901212345678901 -> to the end
+              #     v5          v12         v12       v10         v12
+              # 1234512345678901212345678901212345678901234567890121 -> to the end
               # |    |           |           |         |           |
               # Minor    Start       End     Type      Filesystem  Flags
               # 1          0.016     17.000  primary   ext2        boot
               # 2         17.000  17366.000  extended              
               # 5         17.016  15409.000  logical   ext2        
               # 6      15409.016  17366.000  logical   linux-swap  
-              ($minor, $startMB, $endMB, $partition_type, $fstype, $flags) = unpack("A5 A12 A12 A10 A10 A*", $_);
+              ($minor, $startMB, $endMB, $partition_type, $fstype, $flags) = unpack("A5 A12 A12 A10 A12 A*", $_);
 
             }
 
+
+
             # $fstype may not be set, such as in the case of an extended
             # partition.  parted will also report no fstype, even if an 
             # fstype was chosen at partition time, if there is no filesystem
@@ -421,12 +446,15 @@ sub save_partition_information {
 
             if($flags =~ /type=/) {
                 $id = (split(/=/,$flags))[1];
-                $flags = "";
+                # Exclude the partition type, but preserve other flags. -AR-
+                $flags =~ s/type=[0-9a-fA-F]*//;
+                $flags =~ s/^, //;
+                $flags =~ s/, $//;
             }
 
             unless($id) { $id=""; }
 
-            _print_to_auto_install_conf_file( $minor, $size, $partition_type, $id, $name, $flags );
+            _print_to_auto_install_conf_file( $disk, $minor, $size, $partition_type, $id, $name, $flags );
 
           }
 
@@ -637,6 +665,8 @@ sub _turn_sfdisk_output_into_generic_partitionschemes_file {
      
         } elsif ($id eq "a0") {  # a0  IBM Thinkpad hibernation
 
+        } elsif ($id eq "db") {  # db  CP/M / CTOS / .
+
         } elsif ($id eq "de") {  # de  Dell Utility
 
         } elsif ($id eq "ef") {  # ef  EFI (FAT-12/16/32)
@@ -685,7 +715,7 @@ sub _turn_sfdisk_output_into_generic_partitionschemes_file {
             $size = "*";
         }
 
-        _print_to_auto_install_conf_file( $minor, $size, $partition_type, $id, $name, $flags );
+        _print_to_auto_install_conf_file( $disk, $minor, $size, $partition_type, $id, $name, $flags );
      
       }
     }
@@ -693,10 +723,10 @@ sub _turn_sfdisk_output_into_generic_partitionschemes_file {
 
 
 # Usage:
-# _print_to_auto_install_conf_file($minor, $startMB, $endMB, $partition_type, $id, $name, $flags);
+# _print_to_auto_install_conf_file($disk, $minor, $startMB, $endMB, $partition_type, $id, $name, $flags);
 sub _print_to_auto_install_conf_file {
 
-    my ($minor, $size, $partition_type, $id, $name, $flags) = @_;
+    my ($disk, $minor, $size, $partition_type, $id, $name, $flags) = @_;
 
     # Name may not be set in some cases.
     unless ($name) { $name = "-"; }
@@ -717,10 +747,93 @@ sub _print_to_auto_install_conf_file {
         ("$id" eq "41")  # 41  PPC PReP Boot 
        ) {
         print DISK_FILE qq(  id="$id");
+    } elsif (
+        ("$id" eq "8e") # LVM partition needs lvm_group attribute -AR-
+        ) {
+        my $part;
+        $_ = "$disk";
+        if ((m|\S/(c[0-9]+d[0-9]+)|) or (m|\S/ar[0-9]+/|) or (m|\S/ataraid/|)) {
+            # Hardware RAID device -AR-
+            $part = $disk . 'p' . $minor;
+        } else {
+            # Standard block device -AR-
+            $part = $disk . $minor;
+        }
+        # Get physical volume information -AR-
+        my $cmd = "pvs --noheadings --separator : /dev/$part 2>/dev/null";
+        open (PV_INFO, "$cmd|");
+        unless (eof(PV_INFO)) {
+            my @pv_data = split(/:/, <PV_INFO>);
+            my $vg_name = $pv_data[1];
+            # This partition will become part to the volume group $vg_name -AR-
+            print DISK_FILE qq( lvm_group="$vg_name");
+        }
+        close(PV_INFO);
     }
     print DISK_FILE qq( />\n);
 }
 
+# Usage: 
+# save_lvm_information( $file );
+sub save_lvm_information {
+    
+    my ($file) = @_;
+    
+    # Parse volume group informations -AR-
+    my $cmd = "vgdisplay -c 2>/dev/null";
+    
+    open(VG, "$cmd|") || return undef;
+    unless (eof(VG)) {
+        open(OUT, ">>$file") or die ("FATAL: Couldn't open $file for appending!");
+        print OUT "\n<lvm>\n";
+    
+        foreach my $vg_line (<VG>) {
+            $vg_line =~ s/^  //;
+            my @vg_data = split(/:/, $vg_line);
+            # Volume group name.
+            my $vg_name = $vg_data[0];
+            # Maximum number of logical volumes.
+            my $vg_max_log_vols = $vg_data[4];
+            # Maximum number of physical volumes.
+            my $vg_max_phys_vols = $vg_data[8];
+            # Physical extent size.
+            my $vg_phys_extent_size = $vg_data[12];
+            
+            print OUT "\t<lvm_group name=\"$vg_name\" max_log_vols=\"$vg_max_log_vols\" max_phys_vols=\"$vg_max_phys_vols\" phys_extent_size=\"${vg_phys_extent_size}K\">\n";
+
+            # Print logical volumes informations for this group -AR-
+            $cmd = "lvdisplay -c 2>/dev/null";
+            open(LV, "$cmd|");
+            foreach my $lv_line (<LV>) {
+                $lv_line =~ s/^  //;
+                my @lv_data = split(/:/, $lv_line);
+                
+                my $lv_group_name = $lv_data[1];
+                unless ($lv_group_name eq $vg_name) {
+                    next;
+                }
+                my $lv_dev = $lv_data[0];
+                $lv_dev =~ s/^.*\///;
+                # Logical volume size is doubled in the columnised output. -AR-
+                my $lv_size = $lv_data[6] / 2;
+                
+                print OUT "\t\t<lv name=\"$lv_dev\" size=\"${lv_size}K\" />\n";
+            }
+            
+            close(LV);
+    
+            print OUT "\t</lvm_group>\n";
+        }
+        
+        print OUT "</lvm>\n";
+        close(OUT);
+    } else {
+        # print "DEBUG: No LVM groups defined on this system.\n";
+    }
+        
+    close(VG);    
+
+}
 
 # Usage: 
 # my $label_type = get_disk_label_type($partition_tool, $disk);
