@@ -3,7 +3,7 @@
 #
 #   Copyright (C) 1999-2004 Brian Elliott Finley
 #
-#   $Id: Server.pm,v 1.90 2004/08/10 22:54:24 brianfinley Exp $
+#   $Id: Server.pm 3401 2006-01-10 12:33:53Z arighi $
 #
 #   2004.06.01 Brian Elliott Finley
 #   - add record_image_retrieval_time function
@@ -11,6 +11,8 @@
 #   - add record_image_retrieved_from function
 #   2004.08.10 Brian Elliott Finley
 #   - change quoting on ia64 append line
+#   2005.05.15 Brian Elliott Finley
+#   - added copy_boot_files_from_image_to_shared_dir()
 #
 
 package SystemImager::Server;
@@ -21,7 +23,7 @@ use strict;
 use File::Copy;
 use File::Path;
 use XML::Simple;
-use vars qw($VERSION @mount_points %device_by_mount_point %filesystem_type_by_mount_point);
+use vars qw($VERSION @mount_points %device_by_mount_point %filesystem_type_by_mount_point $disk_no %dev2disk);
 
 $VERSION="SYSTEMIMAGER_VERSION_STRING";
 
@@ -34,17 +36,21 @@ $VERSION="SYSTEMIMAGER_VERSION_STRING";
 #   _imageexists 
 #   _in_script_add_standard_header_stuff 
 #   _read_partition_info_and_prepare_parted_commands 
+#   _read_partition_info_and_prepare_soft_raid_devs -AR- 
+#   _read_partition_info_and_prepare_pvcreate_commands -AR-
+#   _write_lvm_groups_commands -AR-
+#   _write_lvm_volumes_commands -AR-
 #   _write_boel_devstyle_entry
 #   _write_elilo_conf
 #   _write_out_mkfs_commands 
 #   _write_out_new_fstab_file 
 #   _write_out_umount_commands 
 #   add2rsyncd 
+#   copy_boot_files_from_image_to_shared_dir
 #   copy_boot_files_to_boot_media
 #   create_autoinstall_script
 #   create_image_stub 
 #   gen_rsyncd_conf 
-#   get_full_path_to_image_from_rsyncd_conf 
 #   get_image_path 
 #   ip_quad_2_ip_hex
 #   numerically 
@@ -60,6 +66,36 @@ $VERSION="SYSTEMIMAGER_VERSION_STRING";
 ################################################################################
 
 
+sub copy_boot_files_from_image_to_shared_dir {
+
+    use File::Copy;
+    use File::Path;
+
+    shift;
+    my $image                   = shift;
+    my $image_dir               = shift;
+    my $rsync_stub_dir          = shift;
+    my $autoinstall_boot_dir    = shift;
+
+    my $kernel = $image_dir . "/etc/systemimager/boot/kernel";
+    my $initrd = $image_dir . "/etc/systemimager/boot/initrd.img";
+
+    my $file = $image_dir . "/etc/systemimager/boot/ARCH";
+    open(FILE,"<$file") or die("Couldn't open $file for reading $!");
+        my $arch = (<FILE>)[0];
+    close(FILE);
+    chomp $arch;
+
+    my $dir = "$autoinstall_boot_dir/$arch/$image";
+    eval { mkpath($dir, 0, 0755) };
+    if ($@) { print "Couldn’t create $dir: $@"; }
+    copy("$kernel","$dir") or die "Copy failed: $!";
+    copy("$initrd","$dir") or die "Copy failed: $!";
+
+    return 1;
+}
+
+
 sub record_image_retrieved_from {
 
     shift;
@@ -83,6 +119,7 @@ sub record_image_retrieval_time {
 
     my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
     $year += 1900;
+    $mon += 1;
 
     my $file = $image_dir . "/etc/systemimager/IMAGE_RETRIEVAL_TIME";
 
@@ -189,22 +226,29 @@ sub validate_ip_assignment_option {
   return 0;
 }
 
+#
+#   Usage:  my $path = get_image_path( $stub_dir, $imagename );
+#
 sub get_image_path {
-    my ($class,  $stub_dir, $imagename) = @_;
+
+    my $class       = shift;
+    my $stub_dir    = shift;
+    my $imagename   = shift;
 
     open (FILE, "<$stub_dir/40$imagename") or return undef;
     while (<FILE>) {
-	if (/^\s*path\s*=\s*(\S+)\s$/) {
-	    close FILE;
-	    return $1;
-	}
+        if (/^\s*path\s*=\s*(\S+)\s$/) {
+            close FILE;
+            return $1;
+        }
     }
     close FILE;
+
     return undef;
 }
 
 # Usage:
-# my $path = SystemImager::Server->get_image_path( $rsync_stub_dir, $image );
+# my $path = SystemImager::Server->get_image_path( $rsync_stub_dir, $image );   #XXX
 sub get_full_path_to_image_from_rsyncd_conf {
 
     print "FATAL: get_full_path_to_image_from_rsyncd_conf is depricated.\n";
@@ -406,20 +450,28 @@ sub _read_partition_info_and_prepare_parted_commands {
         );
 
         my $devfs_dev = $devfs_map{$dev};
+        $dev2disk{$devfs_dev} = "DISK".$disk_no++;
+        print $out "if [ -z \$DISKORDER ] ; then\n";
+        print $out "  $dev2disk{$devfs_dev}=$devfs_dev\n";
+        print $out "elif [ -z \$$dev2disk{$devfs_dev} ] ; then\n";
+        print $out qq(  echo "Undefined: $dev2disk{$devfs_dev}"\n);
+        print $out "  shellout\n";
+        print $out "fi\n";
+        $devfs_dev = '$'.$dev2disk{$devfs_dev};
 
         print $out "### BEGIN partition $devfs_dev ###\n";
-        print $out qq(echo "Partitioning $devfs_dev..."\n);
-        print $out qq(echo "Old partition table for $devfs_dev:"\n);
+        print $out qq(logmsg "Partitioning $devfs_dev..."\n);
+        print $out qq(logmsg "Old partition table for $devfs_dev:"\n);
         print $out "parted -s -- $devfs_dev print\n\n";
 
         print $out "# Create disk label.  This ensures that all remnants of the old label, whatever\n";
         print $out "# type it was, are removed and that we're starting with a clean label.\n";
         $cmd = "parted -s -- $devfs_dev mklabel $label_type || shellout";
-        print $out qq(echo "$cmd"\n);
+        print $out qq(logmsg "$cmd"\n);
         print $out "$cmd\n\n";
 
         print $out "# Get the size of the destination disk so that we can make the partitions fit properly.\n";
-        print $out qq(DISK_SIZE=`parted -s $devfs_dev print ) . q(| grep 'Disk geometry for' | sed 's/^.*-//g' | sed 's/\..*$//' `) . qq(\n);
+        print $out qq(DISK_SIZE=`parted -s $devfs_dev unit MB print ) . q(| grep 'Disk geometry for' | sed 's/^.*-//g' | sed 's/\..*$//' | sed 's/MB//' `) . qq(\n);
         print $out q([ -z $DISK_SIZE ] && shellout) . qq(\n);
 
         print $out q(if [ "$ARCH" = "alpha" ]; then) . qq(\n);	
@@ -672,8 +724,10 @@ sub _read_partition_info_and_prepare_parted_commands {
             print $out "\n";
 	    
             $part = &get_part_name($dev, $m);
+            $part =~ /^(.*?)(p?\d+)$/;
+            $part = "\${".$dev2disk{$1}."}".$2;
             $cmd = "Creating partition $part.";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             
             print $out qq(START_MB=$startMB{$m}\n);
             print $out qq(END_MB=$endMB{$m}\n);
@@ -692,7 +746,7 @@ sub _read_partition_info_and_prepare_parted_commands {
                 $cmd = qq(parted -s -- $devfs_dev mkpart $p_type{$m} ext2 ) . q($START_MB $END_MB) . qq( || shellout);
 
             }
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
             
             # Leave info behind for the next partition. -BEF-
@@ -734,7 +788,7 @@ sub _read_partition_info_and_prepare_parted_commands {
               ) {  # We're kinda assuming no one names their partitions "-". -BEF-
             
               $cmd = "parted -s -- $devfs_dev name $m $p_name{$m} || shellout\n";
-              print $out "echo $cmd";
+              print $out "logmsg $cmd";
               print $out "$cmd";
             }
             
@@ -748,7 +802,7 @@ sub _read_partition_info_and_prepare_parted_commands {
                     # Parted 1.6.0 doesn't seem to want to tag gpt partitions with lba.  Hmmm. -BEF-
                     if (($flag eq "lba") and ($label_type eq "gpt")) { next; }
                     $cmd = "parted -s -- $devfs_dev set $m $flag on || shellout\n";
-                    print $out "echo $cmd";
+                    print $out "logmsg $cmd";
                     print $out "$cmd";
                 }
             }
@@ -758,14 +812,14 @@ sub _read_partition_info_and_prepare_parted_commands {
         foreach $m (keys %minors_to_remove) {
           print $out "\n# Gotta lose this one (${dev}${m}) to make the disk look right.\n";
           $cmd = "parted -s -- $devfs_dev rm $m  || shellout";
-          print $out qq(echo "$cmd"\n);
+          print $out qq(logmsg "$cmd"\n);
           print $out "$cmd\n";
         }
 
         print $out "\n";
-        print $out qq(echo "New partition table for $devfs_dev:"\n);
+        print $out qq(logmsg "New partition table for $devfs_dev:"\n);
         $cmd = "parted -s -- $devfs_dev print";
-        print $out qq(echo "$cmd"\n);
+        print $out qq(logmsg "$cmd"\n);
         print $out "$cmd\n";
         print $out "### END partition $devfs_dev ###\n";
         print $out "\n";
@@ -773,6 +827,394 @@ sub _read_partition_info_and_prepare_parted_commands {
     }
 }
 
+# Usage:
+# _read_partition_info_and_prepare_soft_raid_devs( $out, $image_dir, $auto_install_script_conf );
+# TODO: spare disks.
+sub _read_partition_info_and_prepare_soft_raid_devs {
+    my ($out, $image_dir, $file) = @_;
+
+    my $xml_config = XMLin($file, keyattr => { lvm_group => "+name" }, forcearray => 1 );
+    unless (defined($xml_config->{raid})) {
+        return;
+    }
+
+    my $cmd;
+    
+    # Remove previous raidtab.
+    $cmd = 'rm -rf /etc/raidtab';
+    print $out qq(logmsg "$cmd"\n);
+    print $out "$cmd\n";
+
+    # Load RAID modules.
+    print $out qq(logmsg "Load software RAID modules."\n);
+    print $out qq(modprobe linear\n);
+    print $out qq(modprobe raid0\n);
+    print $out qq(modprobe raid1\n);
+    print $out qq(modprobe raid5\n);
+
+    # Get all software RAID blocks.
+    foreach my $raid (@{$xml_config->{raid}}) {
+        my @all_devices = get_all_devices($file);
+        my %devfs_map = dev_to_devfs(@all_devices) or return undef;
+
+        # Find the partitions assigned to each RAID disk. -AR-    
+        foreach my $raid_disk (@{$raid->{raid_disk}}) {
+            # Write the command to create the software RAID disk.
+            print $out "cat << EOF >> /etc/raidtab\n";
+
+            # RAID device.
+            my $raid_name = $raid_disk->{name};
+            unless ($raid_name) {
+                print "WARNING: software RAID disk without name!\n";
+                next;
+            }
+            unless ($raid_name =~ /^\/dev\/md/) {
+                print "WARNING: software RAID device name not valid: \"$raid_name\"\n";
+                next;
+            }
+            # RAID level (mandatory).
+            my $raid_level = $raid_disk->{raid_level};
+            unless (defined($raid_level)) {
+                print "WARNING: RAID disk \"$raid_disk->{name}\" doesn't have partitions assigned!\n";
+                next;
+            }
+
+            # Write raidtab details.
+            print $out "raiddev " . $raid_disk->{name} . "\n";
+            print $out "\traid-level\t" . $raid_level . "\n";
+            print $out "\tpersistent-superblock\t" . $raid_disk->{persistent_superblock} . "\n"
+                if ($raid_disk->{persistent_superblock});
+            print $out "\tchunk-size\t" . ($raid_disk->{chunk_size} || 32) . "\n";
+            if ($raid_level == 5) {
+                print $out "\tparity-algorithm\t" . $raid_disk->{parity_algorithm} . "\n"
+                    if ($raid_disk->{parity_algorithm});
+            }
+
+            # Evaluate partitions assigned to the software RAID device.
+            my @part_list;
+
+            foreach my $disk (@{$xml_config->{disk}}) {
+                my $dev = $disk->{dev};
+
+                # Figure out what the highest partition number is.
+                my $highest_part_num = 0;
+                foreach my $part ( @{$disk->{part}} ) {
+                    my $num = $part->{num};
+                    if ($num > $highest_part_num) {
+                        $highest_part_num = $num;
+                    }
+                }
+            
+                # Evaluate the partition list for the current software RAID device.
+                my $m = "0";
+                foreach my $part (@{$disk->{part}}) {
+                    $m++;
+                    unless (defined($part->{raid_dev})) { next; }
+                    if ($part->{raid_dev} eq $raid_disk->{name}) {
+                        if (defined($part->{num})) {
+                            $m = $part->{num};
+                        }
+                        my $part_name = &get_part_name($dev, $m);
+                        if ($part_name =~ /^(.*?)(p?\d+)$/) {
+                            $part_name = "\${".$dev2disk{$1}."}".$2;
+                        }
+                        push(@part_list, $part_name);
+                    }
+                }
+            }
+        
+            if (@part_list) {
+                print $out "\tnr-raid-disks\t" . ($#part_list + 1) . "\n\n";
+                for (my $i = 0; $i <= $#part_list; $i++) {
+                    print $out "\tdevice $part_list[$i]\n\traid-disk\t$i\n";
+                }
+            } else {
+                print "WARNING: RAID disk \"$raid_disk->{name}\" doesn't have partitions assigned!\n";
+            }
+            # Software RAID device defined in /etc/raidtab.
+            print $out "\nEOF\n";
+
+            $cmd = "mkraid --really-force $raid_name || shellout";
+            print $out qq(logmsg "$cmd"\n);
+            print $out "$cmd\n";
+        }
+    }
+
+}
+
+# Usage:
+# _read_partition_info_and_prepare_pvcreate_commands( $out, $image_dir, $auto_install_script_conf );
+sub _read_partition_info_and_prepare_pvcreate_commands {
+    my ($out, $image_dir, $file) = @_;
+
+    my $xml_config = XMLin($file, keyattr => { disk => "+dev", part => "+num" }, forcearray => 1 );
+
+    my @all_devices = get_all_devices($file);
+    my %devfs_map = dev_to_devfs(@all_devices) or return undef;
+    my $cmd;
+
+    foreach my $dev (sort (keys ( %{$xml_config->{disk}} ))) {
+
+        my (
+            $highest_part_num,
+            $m,
+            $part,
+        );
+
+        my $devfs_dev = $devfs_map{$dev};
+
+        ### BEGIN Populate the simple hashes. -BEF- ###
+        my (
+            %flags,
+            %p_type,
+            %p_name,
+        );
+
+        foreach my $m (sort (keys ( %{$xml_config->{disk}->{$dev}->{part}} ))) {
+            $flags{$m}       = $xml_config->{disk}->{$dev}->{part}{$m}->{flags};
+            $p_name{$m}      = $xml_config->{disk}->{$dev}->{part}{$m}->{p_name};
+            $p_type{$m}      = $xml_config->{disk}->{$dev}->{part}{$m}->{p_type};
+        }
+
+        # Figure out what the highest partition number is. -BEF-
+        foreach (sort { $a <=> $b } (keys ( %{$xml_config->{disk}->{$dev}->{part}} ))) {
+            $highest_part_num = $_;
+        }
+
+        $m = "0";
+        until ($m >= $highest_part_num) {
+
+            $m++;
+            unless (defined($p_type{$m})) { next; }
+
+            $part = &get_part_name($dev, $m);
+
+            # Extended partitions can't be used by LVM. -AR-
+            if ("$p_type{$m}" eq "extended") { next; }
+
+            ### Deal with LVM flag for each partition. -AR-
+            if (($flags{$m}) and ($flags{$m} ne "-")) {
+                my @flags = split (/,/, $flags{$m});
+                foreach my $flag (@flags) {
+                    if ("$flag" eq "lvm") {
+                        # Get volume group for this patition -AR-
+                        my $vg_name = $xml_config->{disk}->{$dev}->{part}->{$m}->{lvm_group};
+                        unless (defined($vg_name)) {
+                            print "WARNING: LVM partition \"${dev}${m}\" is not assigned to any group!\n";
+                            next;
+                        }
+                        # Get the version of the LVM metadata to use -AR-
+                        foreach my $lvm (@{$xml_config->{lvm}}) {
+                            my $version = $lvm->{version};
+                            unless (defined($version)) {
+                                # Default => get LVM2 metadata type.
+                                $version = 2;
+                            }
+                            foreach my $lvm_group_name (@{$lvm->{lvm_group}}) {
+                                if ($lvm_group_name->{name} eq $vg_name) {
+                                    $cmd = "Initializing partition $part for use by LVM.";
+                                    print $out qq(logmsg "$cmd"\n);
+
+                                    $cmd = "pvcreate -M${version} -ff -y $part || shellout";
+                                    print $out qq(logmsg "$cmd"\n);
+                                    print $out "$cmd\n";
+                                    goto part_done;
+                                }
+                            }
+                        }
+                    }
+                }
+part_done:
+            }
+        }
+    }
+
+    # Initialize software RAID volumes used for LVM (if present).
+    foreach my $raid (@{$xml_config->{raid}}) { 
+        foreach my $raid_disk (@{$raid->{raid_disk}}) {
+            my $vg_name = $raid_disk->{lvm_group};
+            unless ($vg_name) {
+                next;
+            }
+            my $raid_name = $raid_disk->{name};
+            unless ($raid_name) {
+                next;
+            }
+
+            # Get the version of the LVM metadata to use.
+            foreach my $lvm (@{$xml_config->{lvm}}) {
+                my $version = $lvm->{version};
+                unless (defined($version)) {
+                    # Default => get LVM2 metadata type.
+                    $version = 2;
+                }
+                foreach my $lvm_group_name (@{$lvm->{lvm_group}}) {
+                    if ($lvm_group_name->{name} eq $vg_name) {
+                        $cmd = "pvcreate -M${version} -ff -y $raid_name || shellout";
+                        print $out qq(logmsg "$cmd"\n);
+                        print $out "$cmd\n";
+                    }
+                }
+            }
+        }
+   }
+}
+
+# Usage:  
+# write_lvm_groups_commands( $out, $image_dir, $auto_install_script_conf );
+sub write_lvm_groups_commands {
+    
+    my ($out, $image_dir, $file) = @_;
+
+    my $xml_config = XMLin($file, keyattr => { lvm_group => "+name" }, forcearray => 1 );
+    
+    my $cmd;
+    
+    # Get all LVM blocks.
+    foreach my $lvm (@{$xml_config->{lvm}}) {
+        my @all_devices = get_all_devices($file);
+        my %devfs_map = dev_to_devfs(@all_devices) or return undef;
+        
+        my $version = $lvm->{version};
+        unless (defined($version)) {
+            # Default => get LVM2 metadata type.
+            $version = 2;
+        }
+
+        # Find the partitions assigned to each LVM group. -AR-    
+        foreach my $group_name (sort (keys ( %{$lvm->{lvm_group}} ))) {
+            my $part_list = "";
+
+            foreach my $disk (@{$xml_config->{disk}}) {
+                my $dev = $disk->{dev};
+            
+                # Figure out what the highest partition number is. -AR-
+                my $highest_part_num = 0;
+                foreach my $part ( @{$disk->{part}} ) {
+                    my $num = $part->{num};
+                    if ($num > $highest_part_num) {
+                        $highest_part_num = $num;
+                    }
+                }
+            
+                # Evaluate the partition list for the current LVM group -AR-
+                my $m = "0";
+                foreach my $part (@{$disk->{part}}) {
+                    $m++;
+                    unless (defined($part->{lvm_group})) { next; }
+                    if ($part->{lvm_group} eq $group_name) {
+                        if (defined($part->{num})) {
+                            $m = $part->{num};
+                        }
+                        my $part_name = &get_part_name($dev, $m);
+                        if ($part_name =~ /^(.*?)(p?\d+)$/) {
+                            $part_name = "\${".$dev2disk{$1}."}".$2;
+                        }
+                        $part_list .= " $part_name";
+                    }
+                }
+            }
+
+            # Find RAID disks assigned to the volume group.
+            foreach my $raid (@{$xml_config->{raid}}) {
+                foreach my $raid_disk (@{$raid->{raid_disk}}) {
+                    unless (defined($raid_disk->{lvm_group})) { next; }
+                    if ($raid_disk->{lvm_group} eq $group_name) {
+                        # Add RAID device to the partition list.
+                        my $raid_name = $raid_disk->{name};
+                        if ($raid_name) {
+                            $part_list .= " $raid_name";
+                        } else {
+                            print "WARNING: software RAID disk without name!\n";
+                        }
+                    }
+                }
+            }
+
+            if ($part_list ne "") {
+                # Evaluate the volume group options -AR-
+                my $vg_max_log_vols = $lvm->{lvm_group}->{$group_name}->{max_log_vols};
+                if (defined($vg_max_log_vols)) { 
+                    $vg_max_log_vols = "-l $vg_max_log_vols ";
+                } else {
+                    $vg_max_log_vols = ""; 
+                }
+                my $vg_max_phys_vols = $lvm->{lvm_group}->{$group_name}->{max_phys_vols};
+                if (defined($vg_max_phys_vols)) { 
+                    $vg_max_phys_vols = "-p $vg_max_phys_vols ";
+                } else {
+                    $vg_max_phys_vols = "";
+                }
+                my $vg_phys_extent_size = $lvm->{lvm_group}->{$group_name}->{phys_extent_size};
+                if (defined($vg_phys_extent_size)) { 
+                    $vg_phys_extent_size = "-s $vg_phys_extent_size ";
+                } else {
+                    $vg_phys_extent_size = ""; 
+                }
+                # Remove previous volume groups with $group_name if already present.
+                $cmd = "lvremove -f /dev/${group_name} >/dev/null 2>&1 && vgremove $group_name >/dev/null 2>&1";
+                print $out qq(logmsg "$cmd"\n);
+                print $out "$cmd\n";
+                # Write the command to create the volume group -AR-
+                $cmd = "vgcreate -M${version} ${vg_max_log_vols}${vg_max_phys_vols}${vg_phys_extent_size}${group_name}${part_list} || shellout";
+                print $out qq(logmsg "$cmd"\n);
+                print $out "$cmd\n";
+            } else {
+                print "WARNING: LVM group \"$group_name\" doesn't have partitions!\n";
+            }
+        }
+    }
+}
+
+# Usage:  
+# write_lvm_volumes_commands( $out, $image_dir, $auto_install_script_conf );
+sub write_lvm_volumes_commands {
+    my ($out, $image_dir, $file) = @_;
+
+    my $xml_config = XMLin($file, keyattr => { lvm_group => "+name" }, forcearray => 1 );
+    
+    my $lvm = @{$xml_config->{lvm}}[0];
+    unless (defined($lvm)) {
+        return;
+    }
+    
+    foreach my $group_name (sort (keys ( %{$lvm->{lvm_group}} ))) {
+        
+        foreach my $lv (@{$lvm->{lvm_group}->{$group_name}->{lv}}) {
+        
+            my $cmd;    
+        
+            # Get logical volume name -AR-
+            my $lv_name = $lv->{name};
+            unless (defined($lv_name)) {
+                print "WARNING: undefined logical volume name! skipping volume creation.\n";
+                next;
+            }
+            # Get logical volume size -AR-
+            my $lv_size = $lv->{size};
+            unless (defined($lv_size)) {
+                print "WARNING: undefined logical volume size! skipping volume creation.\n";
+                next;
+            }
+            # Get additional options (expressed in lvcreate format) -AR-
+            my $lv_options = $lv->{lv_options};
+            unless (defined($lv_options)) {
+                $lv_options = "";
+            }
+
+            # Create the logical volume -AR-
+            $cmd = "lvcreate $lv_options -L${lv_size} -n $lv_name $group_name || shellout";
+            print $out qq(logmsg "$cmd"\n);
+            print $out "$cmd\n";
+            
+            # Enable the logical volume -AR-
+            $cmd = "lvscan > /dev/null; lvchange -a y /dev/$group_name/$lv_name || shellout";
+            print $out qq(logmsg "$cmd"\n);
+            print $out "$cmd\n";
+        }
+    }
+}
+
 # Usage:  
 # upgrade_partition_schemes_to_generic_style($image_dir, $config_dir);
 sub upgrade_partition_schemes_to_generic_style {
@@ -871,6 +1313,7 @@ sub _write_out_mkfs_commands {
 
     my @all_devices = get_all_devices($file);
     my %devfs_map = dev_to_devfs(@all_devices) or return undef;
+    my @d2dkeys = reverse sort keys %dev2disk;
 
 
     # Figure out if software RAID is in use. -BEF-
@@ -890,13 +1333,13 @@ sub _write_out_mkfs_commands {
         }
     }
 
-    if ($software_raid or $raidtab) {
+    if (($software_raid or $raidtab) and not($xml_config->{raid})) {
 
 	# XXX at some point, we want to include this in the autoinstallscript.conf
 	# file.  We should also look at the format and write functions for that 
 	# same file. -BEF-
         print $out qq(# /etc/raidtab that will be used for creating software RAID devices on client(s).\n);
-        print $out qq(cat <<'EOF' > /etc/raidtab\n);
+        print $out qq(cat <<EOF > /etc/raidtab\n);
         if (!$raidtab) {
             $raidtab = $image_dir . "/etc/raidtab";
         }
@@ -905,6 +1348,11 @@ sub _write_out_mkfs_commands {
         }
         open(FILE,"<$raidtab") or croak("Couldn't open $raidtab for reading.");
             while (<FILE>) {
+                foreach my $key (@d2dkeys) {
+                    next unless /\Q$key/;
+                    s/\Q$key\E/\${$dev2disk{$key}}/;
+                    last;
+                }
                 print $out $_;
             }
         close(FILE);
@@ -912,9 +1360,9 @@ sub _write_out_mkfs_commands {
         print $out qq(# /etc/raidtab that will be used for creating software RAID devices on client(s).\n);
         print $out qq(\n);
         print $out qq(if [ -e /etc/raidtab ]; then\n);
-		print $out qq(    echo "Ah, good.  Found an /etc/raidtab file.  Proceeding..."\n);
+		print $out qq(    logmsg "Ah, good.  Found an /etc/raidtab file.  Proceeding..."\n);
 		print $out qq(else\n);
-		print $out qq(    echo "No /etc/raidtab file.  Please verify that you have one in your image, or in an override directory."\n);
+		print $out qq(    logmsg "No /etc/raidtab file.  Please verify that you have one in your image, or in an override directory."\n);
 		print $out qq(    shellout\n);
 		print $out qq(fi\n);
         print $out "\n";
@@ -962,7 +1410,12 @@ sub _write_out_mkfs_commands {
 
         # software RAID devices (/dev/md*)
         if ($real_dev =~ /\/dev\/md/) {
-            print $out qq(mkraid --really-force $real_dev || shellout\n);
+            print $out qq(mkraid --really-force $real_dev || shellout\n)
+                unless (defined($xml_config->{raid}));
+        } elsif( $real_dev =~ /^(.*?)(p?\d+)$/ ) {
+            if ($dev2disk{$1}) {
+                $real_dev = "\${".$dev2disk{$1}."}".$2;
+            }
         }
 
         # swap
@@ -970,12 +1423,12 @@ sub _write_out_mkfs_commands {
 
             # create swap
             $cmd = "mkswap -v1 $real_dev || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
 
             # swapon
             $cmd = "swapon $real_dev || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
 
             print $out "\n";
@@ -985,17 +1438,17 @@ sub _write_out_mkfs_commands {
 
             # create fs
             $cmd = "mkdosfs $mkfs_opts -v $real_dev || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
 
             # mkdir
             $cmd = "mkdir -p /a$mp || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
 
             # mount
             $cmd = "mount $real_dev /a$mp -t $fs -o $options || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
             
             print $out "\n";
@@ -1006,7 +1459,7 @@ sub _write_out_mkfs_commands {
 
             # create fs
             $cmd = "mke2fs $real_dev || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
 
             if ($mount_dev) {
@@ -1016,7 +1469,7 @@ sub _write_out_mkfs_commands {
                     $label =~ s/LABEL=//;
                 
                     $cmd = "tune2fs -L $label $real_dev";
-                    print $out qq(echo "$cmd"\n);
+                    print $out qq(logmsg "$cmd"\n);
                     print $out "$cmd\n";
                 }
                 
@@ -1026,19 +1479,19 @@ sub _write_out_mkfs_commands {
                     $uuid =~ s/UUID=//;
                 
                     $cmd = "tune2fs -U $uuid $real_dev";
-                    print $out qq(echo "$cmd"\n);
+                    print $out qq(logmsg "$cmd"\n);
                     print $out "$cmd\n";
                 }
             }
 
             # mkdir
             $cmd = "mkdir -p /a$mp || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
 
             # mount
             $cmd = "mount $real_dev /a$mp -t $fs -o $options || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
             
             print $out "\n";
@@ -1049,7 +1502,7 @@ sub _write_out_mkfs_commands {
 
             # create fs
             $cmd = "mke2fs -j $real_dev || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
 
             if ($mount_dev) {
@@ -1059,7 +1512,7 @@ sub _write_out_mkfs_commands {
                     $label =~ s/LABEL=//;
                 
                     $cmd = "tune2fs -L $label $real_dev";
-                    print $out qq(echo "$cmd"\n);
+                    print $out qq(logmsg "$cmd"\n);
                     print $out "$cmd\n";
                 }
                 
@@ -1069,19 +1522,19 @@ sub _write_out_mkfs_commands {
                     $uuid =~ s/UUID=//;
                 
                     $cmd = "tune2fs -U $uuid $real_dev";
-                    print $out qq(echo "$cmd"\n);
+                    print $out qq(logmsg "$cmd"\n);
                     print $out "$cmd\n";
                 }
             }
 
             # mkdir
             $cmd = "mkdir -p /a$mp || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
 
             # mount
             $cmd = "mount $real_dev /a$mp -t $fs -o $options || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
             
             print $out "\n";
@@ -1091,18 +1544,18 @@ sub _write_out_mkfs_commands {
         } elsif ( $xml_config->{fsinfo}->{$line}->{fs} eq "reiserfs" ) {
 
             # create fs
-            $cmd = "echo y | mkreiserfs $real_dev || shellout";
-            print $out qq(echo "$cmd"\n);
+            $cmd = "mkreiserfs -q $real_dev || shellout";
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
 
             # mkdir
             $cmd = "mkdir -p /a$mp || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
 
             # mount
             $cmd = "mount $real_dev /a$mp -t $fs -o $options || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
             
             print $out "\n";
@@ -1112,17 +1565,17 @@ sub _write_out_mkfs_commands {
 
             # create fs
             $cmd = "jfs_mkfs -q $real_dev || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
 
             # mkdir
             $cmd = "mkdir -p /a$mp || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
 
             # mount
             $cmd = "mount $real_dev /a$mp -t $fs -o $options || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
             
             print $out "\n";
@@ -1132,17 +1585,17 @@ sub _write_out_mkfs_commands {
 
             # create fs
             $cmd = "mkfs.xfs -f -q $real_dev || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
 
             # mkdir
             $cmd = "mkdir -p /a$mp || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
 
             # mount
             $cmd = "mount $real_dev /a$mp -t $fs -o $options || shellout";
-            print $out qq(echo "$cmd"\n);
+            print $out qq(logmsg "$cmd"\n);
             print $out "$cmd\n";
             
             print $out "\n";
@@ -1302,6 +1755,7 @@ sub _write_out_umount_commands {
                    or ($fs eq "jfs")
                    or ($fs eq "xfs")
                    or ($fs eq "proc")
+                   or ($fs eq "sysfs")
             ) { next; }
 
             # 
@@ -1317,6 +1771,7 @@ sub _write_out_umount_commands {
     # the <fsinfo> section of the autoinstallscript.conf file.
     #
     $fs_by_mp{'/proc'} = "proc";
+    $fs_by_mp{'/sys'} = "sysfs";
 
     #
     # If client uses devfs, then unmount the bound /dev filesystem.
@@ -1336,14 +1791,36 @@ sub _write_out_umount_commands {
 
         # umount
         my $cmd = "umount /a$mp || shellout";
-        print $out qq(echo "$cmd"\n);
+        print $out qq(if [ ! \$kernel = "2.4" ]; then\n) if ($mp eq "/sys");
+        print $out qq(logmsg "$cmd"\n);
         print $out "$cmd\n";
+        print $out "fi\n" if ($mp eq "/sys");
         print $out "\n";
     }
 }
 
+sub show_disk_edits{
+    my ($out) = shift;
+    foreach (sort keys %dev2disk) {
+        print $out qq(  echo " $_ -> \$$dev2disk{$_}"\n);
+    }
+}
+sub edit_disk_names{
+    my ($out) = shift;
+    foreach (reverse sort keys %dev2disk) {
+        print $out qq(    sed s:$_:%$dev2disk{$_}%:g |\n);
+    }
+    for (my $i = 0; $i < scalar keys %dev2disk; $i++) {
+        print $out qq(    sed s:%DISK$i%:\$DISK$i:g |\n);
+    }
+}
+
 sub write_sc_command {
     my ( $out, $ip_assignment_method ) = @_;
+
+    # Configure the network device used to contact the image-server -AR-
+    print $out "[ -z \$DEVICE ] && DEVICE=eth0\n";
+    
     my $sc_excludes_to = "/etc/systemimager/systemconfig.local.exclude";
     my $sc_cmd = "chroot /a/ systemconfigurator --excludesto=$sc_excludes_to";
     if ($ip_assignment_method eq "replicant") {
@@ -1422,6 +1899,9 @@ sub create_autoinstall_script{
     open (my $TEMPLATE, "<$template") || die "Can't open $template for reading\n";
     open (my $MASTER_SCRIPT, ">$file") || die "Can't open $file for writing\n";
 
+    $disk_no = 0;
+    %dev2disk = ();
+
     my $delim = '##';
     while (<$TEMPLATE>) {
         SWITCH: {
@@ -1447,7 +1927,35 @@ sub create_autoinstall_script{
 	            last SWITCH;
 	        }
 
-	        if (/^\s*${delim}CREATE_FILESYSTEMS${delim}\s*$/) {
+                if (/^\s*${delim}CREATE_SOFT_RAID_DISKS${delim}\s*$/) { 
+                _read_partition_info_and_prepare_soft_raid_devs( $MASTER_SCRIPT,
+                    $image_dir, 
+                    $auto_install_script_conf);
+                    last SWITCH;
+                }
+
+                if (/^\s*${delim}INITIALIZE_LVM_PARTITIONS${delim}\s*$/) {
+ 	            _read_partition_info_and_prepare_pvcreate_commands( $MASTER_SCRIPT,
+ 	          						$image_dir,
+ 	          						$auto_install_script_conf);
+                     last SWITCH;
+                }
+
+                if (/^\s*${delim}CREATE_LVM_GROUPS${delim}\s*$/) {
+                    write_lvm_groups_commands( $MASTER_SCRIPT,
+                                            $image_dir,
+                                            $auto_install_script_conf);
+                     last SWITCH;
+                }
+
+                if (/^\s*${delim}CREATE_LVM_VOLUMES${delim}\s*$/) {
+                     write_lvm_volumes_commands( $MASTER_SCRIPT,
+                                            $image_dir,
+                                            $auto_install_script_conf);
+                     last SWITCH;
+                }
+	        
+                if (/^\s*${delim}CREATE_FILESYSTEMS${delim}\s*$/) {
 	            _write_out_mkfs_commands( $MASTER_SCRIPT, 
 	          			$image_dir, 
 	          			$auto_install_script_conf,
@@ -1501,6 +2009,16 @@ sub create_autoinstall_script{
                 }
                 last SWITCH;
 	        }
+
+			if (/^\s*${delim}SHOW_DISK_EDITS${delim}\s*$/) {
+				show_disk_edits( $MASTER_SCRIPT );
+				last SWITCH;
+			}
+			if (/^\s*${delim}EDIT_DISK_NAMES${delim}\s*$/) {
+				edit_disk_names( $MASTER_SCRIPT );
+				last SWITCH;
+			}
+
 	        ### END end of autoinstall options ###
 	        print $MASTER_SCRIPT $_;
         }
@@ -1738,7 +2256,7 @@ sub _write_boel_devstyle_entry {
     if( defined($xml_config->{boel}->{devstyle}) && ("$xml_config->{boel}->{devstyle}" eq "devfs")) {
 
         my $cmd = q(mount /dev /a/dev -o bind || shellout);
-        print $script qq(echo "$cmd"\n);
+        print $script qq(logmsg "$cmd"\n);
         print $script qq($cmd\n);
         
     } else {
@@ -1749,4 +2267,4 @@ sub _write_boel_devstyle_entry {
 }
 
 
-
+# /* vi: set filetype=perl ai et ts=4 sw=4: */
